SystemOrganization addCategory: #'FLOdactic-Core'!

Object subclass: #DidacticMaterial
	instanceVariableNames: 'authors contributors tags uri title created modified description type license status owner tagsManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

DidacticMaterial class
	instanceVariableNames: 'owners'!

DidacticMaterial class
	instanceVariableNames: 'owners'!

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:51'!
addAuthor: anAuthor
	"Adds anAuthor to the authors"
	
	"author := anAuthor."
	authors add: (anAuthor name) -> anAuthor.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:51'!
addContributor: aContributor
	"Adds a contributor to the contributors, granting with editor permissions"

	contributors add: aContributor name -> aContributor.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:51'!
addTag: aTagString
	"Adds aTagString to the tags"

	| tag testIncludedTag |
	
	testIncludedTag := tagsManager includesKey: aTagString.
	"If it exists on the manager, get it"
	testIncludedTag ifTrue: [
		tag := tagsManager at: aTagString.
	]
	"else, add it to the manager"
	ifFalse: [
		tag := TagObject new.
		tag name: aTagString.
		tagsManager add: aTagString -> tag.
	].
	"finally add it to the tags object Dictionary itself"
	tags add: aTagString -> tag.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:52'!
authors
	"Returns the author"
	
	^ authors.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 14:22'!
catalogCard
	"Prints on the Transcript the catalog card of the didactic material"
	"Reference: http://www.biblioteca.bauru.unesp.br/?_escaped_fragment_=/servicos/ficha-catalografica/"
	| authorsData counter |

	authorsData := ''.	
	authors = nil ifFalse: [
		authors do: [ :author | Transcript show: author name; crlf ].
		Transcript tab; show: title, ', ', modified year; crlf.
		counter := 0.
		tags do: [ :tag | counter := counter+1.
			Transcript show: counter printString, '. ', tag name; space ].
		Transcript crlf.
		Transcript crlf.
	]
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:31'!
description
	^ description.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:40'!
description: aString 
	description := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:53'!
doBlock
	"Blocks the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."
	
	status := 'blocked'.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:53'!
doDisable
	"Disables the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."
	
	status := 'disabled'.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:53'!
doEnable
	"Enables the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."

	status := 'enabled'.
	self touch.! !

!DidacticMaterial methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:01'!
initialize
	"Class responsability: takes cares of a didactic material entity"
	super initialize.
	
	"No attribute is mandatory at this point, everything will be checked when it will be added"
	uri := nil.
	title := nil.
	description := nil.
	type := nil.
	license := nil.
	status := nil.
	
	"Agregations"
"	author := nil."
	contributors := Dictionary new.
	authors := Dictionary new.
	tags := Dictionary new.
	tagsManager := TagsManager singleton.
"	owners := nil."
	
	
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:53'!
isBlocked
	"checks if the didactic material is blocked"
	
	^ status = 'blocked'! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:55'!
isDisabled
	"Checks if the didactic material is disabled"
	
	^ status = 'disabled'! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:55'!
isEnabled
	"Checks if the didactic material is enabled"
	
	^ status = 'enabled'! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:33'!
license
	^ license.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:40'!
license: aString 
	license := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/10/2013 18:04'!
matchesAuthorsByName: anExpression
	"Returns true if at least one of the authors name matches anExpression"

	| list |
	list := self selectAuthorsByName: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:58'!
matchesByDescription: anExpression
	"Returns true if the didactic material description matches anExpression"

	^ anExpression match: description.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:58'!
matchesByOwnerEmail: anExpression
	"Returns true if the didactic material owner's email matches anExpression"

	^ anExpression match: (owner email).
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:58'!
matchesByOwnerName: anExpression
	"Returns true if the didactic material owner's name matches anExpression"

	^ anExpression match: (owner name).
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesByStatus: anExpression
	"Returns true if the didactic material status matches anExpression"

	^ anExpression match: status.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesByTitle: anExpression
	"Returns true if the didactic material title matches anExpression"

	^ anExpression match: title.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesByType: anExpression
	"Returns true if the didactic material type matches anExpression"

	^ anExpression match: type.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/11/2013 16:06'!
matchesContributorsByName: anExpression
	"Returns true if at least one of the contributors name matches anExpression"

	| list |
	list := self selectContributorsByName: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesContributorsByUsername: anExpression
	"Returns true if at least one of the contributors username matches anExpression"

	| list |
	list := self selectContributorsByUsername: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 17:00'!
matchesTagByName: anExpression
	"Returns true if at least one of the tags name matches anExpression"

	| list |
	list := self selectTagsByName: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/10/2013 19:52'!
owner
	^ owner! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/7/2013 05:39'!
owner: anUser 
	| username |
	username := anUser username.

"	(owners includesKey: username) ifTrue: [
		msg := 'User *', username, '* is already an owner for that material'."
		"counter := items at: aTagString."
		"tags add: aTagString -> (counter + 1)."
"	].
	[(owners at: username)] ifError: [
		msg := 'User *', username, '* has been added as owner'.
		owners add: username -> anUser.
	].
	Transcript show: msg; cr."
	owner := anUser.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/10/2013 16:13'!
selectAuthorsByName: anExpression
	"Selects all the authors whose names matches anExpression"

"	^ authors select: [ :author | 
		author name ifNotNil: [
			anExpression match: (author name)
		]
		ifNil: [
			false.
		]
	]."

	^ authors select: [ :author | anExpression match: (author name)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:56'!
selectContributorsByName: anExpression
	"Selects all the contributors whose names matches anExpression"

	^ contributors select: [ :contributor | anExpression match: (contributors name)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:56'!
selectContributorsByUsername: anExpression
	"Selects all the contributor whose usernames matches anExpression"

	^ contributors select: [ :contributor | anExpression match: (contributor username)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/11/2013 16:51'!
selectTagsByName: anExpression
	"Selects tags whose names matches anExpression (could be partly)"

	^ tags select: [ :tag | anExpression match: (tag name)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:33'!
status
	^ status.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:57'!
tags
	"Return the tags associated with the didactic materials as a Dictionary"

	^ tags.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:31'!
title
	^ title.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:40'!
title: aString 
	title := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:36'!
touch
	"Modifies the latest modification timestamp (similar to the touch *nix command)"

	modified := DateAndTime now.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:32'!
type
	^ type.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:34'!
type: aString 
	type := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:29'!
uri
	^ uri.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:34'!
uri: aString 
	uri := aString.
	self touch.! !

Object subclass: #DidacticMaterialsManager
	instanceVariableNames: 'items tags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

DidacticMaterialsManager class
	instanceVariableNames: 'uniqueInstance'!

DidacticMaterialsManager class
	instanceVariableNames: 'uniqueInstance'!

!DidacticMaterialsManager class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:04'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of DidacticMaterialManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/4/2013 11:56'!
addAll: aDictionary
	"Adds all the items on aDictionary to the internal structure"
	"TODO: check for possible errors (duplicated, missing attributes, etc)"

	items := aDictionary.! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/12/2013 14:31'!
catalogCards
	"Prints on the Transcript the catalog cards of each didactic material"

	items do: [ :didacticMaterial | didacticMaterial catalogCard ].! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:02'!
initialize
	"Class responsability: handle didactic materials associations with other objects (authors, contributors, tags, etc)"
	
	super initialize.
	items := Dictionary new.
	Transcript show: 'DidacticMaterialManager initialize'; cr.! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/6/2013 19:00'!
items
	"Returns a Dictionary with the items within the structure.
	This returns a very deep copy of the actual items so modifications doesn't compromise the actual data, nor creates unwanted side effects."

	^ items veryDeepCopy.! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/11/2013 21:58'!
owner: anOwner add: aDidacticMaterial
	"Adds aDidacticMaterial to the internal structure. It also links that didactic material to anAuthor"
	self validate: aDidacticMaterial.
	
	Transcript show: 'The didactic material: *', aDidacticMaterial uri, '* DOESNT exists!!'; cr.
	aDidacticMaterial owner: anOwner.
	aDidacticMaterial doEnable.
	aDidacticMaterial touch.
	items add: aDidacticMaterial uri -> aDidacticMaterial.
	! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/11/2013 19:17'!
owner: anOwner edit: aDidacticMaterial
	"Edits aDidacticMaterial."
	self validate: aDidacticMaterial.
	
	Transcript show: 'Editing: *', aDidacticMaterial uri, '*'; cr.
	aDidacticMaterial touch.
	items at: (aDidacticMaterial uri) put: aDidacticMaterial.
	! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/10/2013 14:03'!
remove
	self removeAll.! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 22:35'!
remove: aDidacticMaterial
	"Removes from the structure anUriString. Raises errors on:
	- Non existing anUriString (verified with...)"
	
	| item test msg anUriString |
	anUriString := aDidacticMaterial uri.
	test := items includesKey: anUriString.
	(test) ifTrue: [
		msg := 'The didactic material exists on the system, removing it...'.
		Transcript show: msg; crlf.
		item := items at: anUriString.
		items removeKey: anUriString.
	].
	(test) ifFalse: [
		msg := 'The didactic material doesnt exists on the system, cannot remove it...'.
		Transcript show: msg; crlf.
		"self error: msg"

	].! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/10/2013 14:56'!
removeAll
	"Our dirty solution to avoid persistance in Squeak. In order to have the tests passing, we need to remove all the items on each test. This is not intended to be used by the API user"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/4/2013 11:58'!
select: aFilter
	"Returns a structure (Dictionary) of filtered items by aFilter criteria
	This should emulates something like 'list comprehensions' in python. Cool we manage to achieve this!! =D
	"

	| result |
	result := DidacticMaterialsManager new.
	result addAll: (items select: aFilter).
	^ result.
	! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/4/2013 10:16'!
size
	"Returns the size of the internal structure"

	^ items size.! !

!DidacticMaterialsManager methodsFor: 'testing' stamp: 'it 5/11/2013 19:04'!
validate: aDidacticMaterial
	"validates the material"
	
	(aDidacticMaterial uri)
		ifNil: [
			ValidationError signal: 'URI cannot be empty!!'.
		] 
		ifNotNil: [
			(items includesKey: (aDidacticMaterial uri))
				ifTrue: [
					"There is a DidacticMaterial with the same URI
					Checks if its not the same instance. Raise an error only if it's a different material"
					(aDidacticMaterial ~= (items at: (aDidacticMaterial uri)))
						ifTrue: [
							ValidationError signal: 'Another didactic material with URI *', aDidacticMaterial uri, '* already exists!!'.
						]
				]
		].

	(aDidacticMaterial title) ifNil: [
		ValidationError signal: 'Title cannot be empty!!'.
	].

	(aDidacticMaterial description) ifNil: [
		ValidationError signal: 'Description cannot be empty!!'.
	].

	(aDidacticMaterial license) ifNil: [
		ValidationError signal: 'License cannot be empty!!'.
	].

	(aDidacticMaterial type) ifNil: [
		ValidationError signal: 'Type cannot be empty!!'.
	].
! !

Object subclass: #FLOdacticFacade
	instanceVariableNames: 'users didacticMaterials tags loggedUser floogle search'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

FLOdacticFacade class
	instanceVariableNames: 'uniqueInstance'!

FLOdacticFacade class
	instanceVariableNames: 'uniqueInstance'!

!FLOdacticFacade class methodsFor: 'as yet unclassified' stamp: 'afv 5/11/2013 12:04'!
newTestsInstance
	^ self basicNew initializeWithoutSingletons.! !

!FLOdacticFacade class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:03'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of FLOdacticFacade...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/10/2013 20:13'!
addContributor: anUser didacticMaterial: aDidacticMaterial 
	aDidacticMaterial addContributor: anUser.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/10/2013 14:54'!
addMaterial: aDidacticMaterial 
	"Adds a didactic material. The owner will be the logged in user"
	
	| test |
	test := self loggedInOrRaiseErrorIfNot.
	test ifTrue: [
		aDidacticMaterial touch.
		didacticMaterials owner: loggedUser add: aDidacticMaterial.
	]! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/9/2013 20:36'!
addUser: anUser 
	"Add an user
	This message doesn't needs login. This emulates self sign in to the system"
	
	users add: anUser.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:35'!
allDidacticMaterials
	"Returns a copy of all didactic materials"
	
	^ didacticMaterials veryDeepCopy.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:35'!
allTags
	"Returns a copy of all tags"
	
	^ tags veryDeepCopy.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:35'!
allUsers
	"Returns a copy of all users"

	^ users veryDeepCopy.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'afv 5/11/2013 23:30'!
canDisable: anObject
	"Returns true only if loggedUser can disable anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			moderators
			administrators
		- an User instance in which case allowed user actors are:
			administrators
	Possible errors:
		User not logged in (nil)"

	| result isDidacticMaterial isUser |
	result := self checkLoggedInOrError.
	result := false.
	isDidacticMaterial := anObject class = DidacticMaterial.
	isDidacticMaterial ifTrue: [
		result := (loggedUser isAdmin) or: (loggedUser isModerator).
	].
	isUser := anObject class = User.
	isUser ifTrue: [
		result := loggedUser isAdmin.
	].
	^ result.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'afv 5/11/2013 22:30'!
canEdit: anObject
	"Returns true if loggedUser can edit anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			owner
			moderators
			administrators
		- an User instance in which case allowed user actors are:
			owner
			administrators
	Possible errors:
		User not logged in (nil)"

	| result isMaterial canDelete |
	result := false.
	canDelete := self canRemove: anObject. "handle errors"
	isMaterial := anObject class = DidacticMaterial.
	isMaterial ifTrue: [
		result := (anObject owner) username = loggedUser username.
		"Verify if loggedUser is a contributor"
		result ifFalse: [
			result := anObject matchesContributorsByUsername: (loggedUser username).
		].
	].
	result := (result) or: (canDelete).
	^ result.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'afv 5/12/2013 00:59'!
canEnable: anObject
	"Returns true only if loggedUser can enable anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			administrators
		- an User instance in which case allowed user actors are:
			administrators
	Possible errors:
		User not logged in (nil)"

	| result |
	result := self checkLoggedInOrError.
	result := loggedUser isAdmin.
	^ result.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'afv 5/11/2013 22:28'!
canRemove: anObject
	"Returns true if loggedUser can remove anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			owner
			moderators
			administrators
		- an User instance in which case allowed user actors are:
			owner (deletes it's own account)
			administrators
	Possible errors:
		User not logged in (nil)"

	| result isDidacticMaterial isUser |
	result := self checkLoggedInOrError.
	result := false.
	isDidacticMaterial := anObject class = DidacticMaterial.
	isDidacticMaterial ifTrue: [
		result := (anObject owner) username = loggedUser username.
		result ifFalse: [
			result := loggedUser isModerator.
		].
		result ifFalse: [
			result := loggedUser isAdmin.
		]
	].
	isUser := anObject class = User.
	isUser ifTrue: [
		result := (anObject username = loggedUser username) or: (loggedUser isAdmin).
	].
	^ result.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'afv 5/11/2013 21:48'!
checkLoggedInOrError
	"comment stating purpose of message"

	| result |
	result := loggedUser ~= nil.
	result ifFalse: [
		self error: 'You might wanna log in fist buddy'.
	].
	^ result.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 5/11/2013 19:42'!
doLogin: aUsernameString password: aPasswordString
	"Performs the login to the system.
	Returns true if login was successfull, false if not
	Possible errors:
	- Already logged in
	- Wrong username or username doesn't exist (delegated to UsersManager class)
	- Wrong password (delegated to UsersManager class)
	- User disabled (delegated to UsersManager class)
	"

	| result msg error |
	
	error := false.
	loggedUser ifNil: [
		loggedUser := users doLogin: aUsernameString password: aPasswordString.
		msg := 'Welcome back to FLOdactic', aUsernameString.
	]
	ifNotNil: [
		msg := 'Someone is already loggedin, logoff if this is not you!!'.
		error := true.
	].
	result := loggedUser username = aUsernameString.
	Transcript show: msg; cr.
	error ifTrue: [
		PermissionError signal: msg.
	].
	^ result.
	
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/9/2013 21:15'!
doLogout
	"comment stating purpose of message"

	loggedUser := nil.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/11/2013 20:32'!
editDidacticMaterial: aDidacticMaterial 
	"Edit a didactic material. The owner must match the logged in user"
	
	| test |
	test := self loggedInOrRaiseErrorIfNot.
	test ifTrue: [
		test := aDidacticMaterial owner ~= nil. "if it already exists..."
		test ifTrue: [
			test := loggedUser username ~= nil.
			test ifTrue: [
				test := self canEdit: aDidacticMaterial.
				test ifTrue: [
					didacticMaterials owner: loggedUser edit: aDidacticMaterial.
				]
			]
		]
	]! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/11/2013 21:40'!
editUser: aNewUser
	"If the username matches and the user is logged in then we accept aNewUser as the updated object
	This is the best way we came arround to allow updates, we don't feel it would be nice to pass every
	property on the same message, so we pass a full object."

	| loggedIn matches user |
	loggedIn := self loggedInOrRaiseErrorIfNot.
	loggedIn ifTrue: [
		matches := (loggedUser username = aNewUser username) or: (self canEdit: aNewUser).
		matches ifTrue: [
			user := users items at: (aNewUser username).
			users items at: (aNewUser username) put: aNewUser.
		]
	]! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/11/2013 22:12'!
feedbackBlock: anObject
	"Changes the status of anObject to locked so admin or moderators can review the object (user
	or didactic material), and if so block it or not"
	
	anObject doBlock.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 00:57'!
feedbackDisable: anObject
	"Changes the status of anObject to disabled so admin or moderators can review the object (user
	or didactic material), and if so block it or not"
	
	(self canDisable: anObject) ifTrue: [
		anObject doDisable.
	].! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 01:01'!
feedbackEnable: anObject
	"Changes the status of anObject to enabled so admin or moderators can review the object (user
	or didactic material), and if so block it or not"
	
	(self canEnable: anObject) ifTrue: [
		anObject doEnable.
	].! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:03'!
initialize
	"Class responsability: Abstract a session of a user on the FLOdactic system and centralize all user's actions on the system within a single access interface"

	users := UsersManager singleton.
	didacticMaterials := DidacticMaterialsManager singleton.
	tags := TagsManager singleton.
	
	search := SearchEngine new.
	
	Transcript show: 'Creating users interface...'; crlf.
	^ self! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'it 5/11/2013 21:23'!
initializeWithoutSingletons
	"This is a method only for use with the tests suite, in order to recreate objects all the time"

	users := UsersManager new.
	didacticMaterials := DidacticMaterialsManager new.
	tags := TagsManager new.
	
	search := SearchEngine new.
	"Need to set the same Managers into the SearchEngine, otherwise tests will fail"
	search didacticMaterialsManager: didacticMaterials.
	search usersManager: users.
	search tagsManager: tags.
	
	Transcript show: 'Creating users interface...'; crlf.
	^ self! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'afv 5/9/2013 21:29'!
loggedInOrRaiseErrorIfNot
	"Checks if the user is logged in, and if not raises an error"

	| msg test |
	test := loggedUser = nil.
	test ifTrue: [ 
		msg := 'Not logged in, you must log in first in order to complete that action. No cookies for you my friend!!'.
		self error: msg; cr.
	].
	^ test not.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:36'!
loggedUser
	"Returns the logged user"
	
	^ loggedUser veryDeepCopy.! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/9/2013 19:54'!
remove
	search remove.
	
	didacticMaterials remove.
	users remove.
	tags remove.! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'it 5/11/2013 18:26'!
removeAll
	"This is for testing purposes, persistent objects within tests are breaking some of the tests"

	users removeAll.
	users release.
	
	didacticMaterials removeAll.
	didacticMaterials release.

	tags removeAll.
	tags release.! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 22:34'!
removeDidacticMaterial: aDidacticMaterial
	(self canRemove: aDidacticMaterial) ifTrue: [
		didacticMaterials remove: aDidacticMaterial.
	].! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 22:28'!
removeUser: anUsernameString 
	users remove: anUsernameString.! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 12:23'!
resetInstance
	"We couldn't sucessfuly delete objects in Squek so we reset the instance and it's attributes.
	Unelegant, we know, but we couln't find other better way"

	users removeAll..
	didacticMaterials removeAll.
	tags removeAll.
	
	users := nil.
	didacticMaterials := nil.
	tags := nil.
	search := nil.
	
	Transcript show: 'Clearing users interface...'; crlf.
	^ self! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 6/8/2013 10:22'!
searchDidacticMaterials: aStringExpression
	"Returns an ordered collection with the search results by aStringExpression,
	by default only enabled documents will be shown"

	| results filter |
	results := search searchDidacticMaterials: aStringExpression.
	((loggedUser isAdmin) or: (loggedUser isModerator)) ifTrue: [	
		filter := search filterAll.
	]
	ifFalse: [
		filter := Filter didacticMaterialsByStatus: 'enabled'.
	].
	results := search addFilter: filter.
	^ results! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 5/11/2013 14:41'!
searchTags: aStringExpression
	"Returns an ordered collection with the search results by aStringExpression"

	^ search searchTags: aStringExpression.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/9/2013 22:13'!
searchUsers: aStringExpression
	"Returns an ordered collection with the search results by aStringExpression"

	^ search searchUsers: aStringExpression.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 14:31'!
transcriptCatalogCards
	"Prints on the transcript a catalog card for each didactic material"

	didacticMaterials catalogCards.
"	didacticMaterials do: [ :didacticMaterial | didacticMaterial catalogCard ]."
	! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/2/2013 18:03'!
usersAsOrderedCollection
	^ users asOrderedCollection.! !

Object subclass: #Filter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 09:51'!
defaultFilterPolicy: anExpressionString
	"This is the default policy for searching message
	We decided to wrap the expression with wildcards as default, this can be easily changed here"

	^ self wildCardExpression: anExpressionString.! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:25'!
didacticMaterialsAll
	"This filter expression doesn't filter anything, it will be used to return the results unmodified on the admin / moderators searches"
	| |

	^ [ :didacticMaterial | true].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 10:11'!
didacticMaterialsByAuthorName: anAuthorNameString
	"Filter expression by Author's name"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: anAuthorNameString.
	searchExpression := self defaultFilterPolicy: anAuthorNameString.
	^ [ :didacticMaterial | didacticMaterial matchesAuthorsByName: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:07'!
didacticMaterialsByDescription: aDescriptionString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aDescriptionString.
	searchExpression := self defaultFilterPolicy: aDescriptionString.
	^ [ :didacticMaterial | didacticMaterial matchesByDescription: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:07'!
didacticMaterialsByOwnerEmail: anOwnerString
	"Didactic material filter expression by owner"
	| searchExpression |
	
	self isSearchingAString: anOwnerString.
	searchExpression := self defaultFilterPolicy: anOwnerString.
	^ [ :didacticMaterial | didacticMaterial matchesByOwnerEmail: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByOwnerName: anOwnerString
	"Didactic material filter expression by owner"
	| searchExpression |
	
	self isSearchingAString: anOwnerString.
	searchExpression := self defaultFilterPolicy: anOwnerString.
	^ [ :didacticMaterial | didacticMaterial matchesByOwnerName: searchExpression].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 10:07'!
didacticMaterialsByStatus: aStatusString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aStatusString.
	searchExpression := self defaultFilterPolicy: aStatusString.
	^ [ :didacticMaterial | didacticMaterial matchesByStatus: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByTag: aTagString
	"Didactic material filter expression by Author's name"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aTagString.
	searchExpression := self defaultFilterPolicy: aTagString.
	^ [ :didacticMaterial | didacticMaterial matchesTagByName: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByTitle: aTitleString
	"Filter expression by Title"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aTitleString.
	searchExpression := self defaultFilterPolicy: aTitleString.
	^ [ :didacticMaterial | didacticMaterial matchesByTitle: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByType: aTypeString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aTypeString.
	searchExpression := self defaultFilterPolicy: aTypeString.
	^ [ :didacticMaterial | didacticMaterial matchesByType: searchExpression].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 10:23'!
isSearchingAString: input
	"Checks if the filter's input parameter is null/empty and raises an error if so"
	| isInvalid |
	(isInvalid := input isString not) ifFalse: [
		isInvalid := input isEmpty
	].

	isInvalid ifTrue: [self error: 'Invalid filter expression. It must be a string'].
	^ isInvalid! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:05'!
usersByRole: aRoleString
	"Users filtered by type"
	| searchExpression |
	
	self isSearchingAString: aRoleString.
	searchExpression := self defaultFilterPolicy: aRoleString.
	^ [ :user | user matchesByRole: searchExpression].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/7/2013 12:11'!
wildCardExpression: anExpression
	"Returns an expression surrounded by asterisk (wildcard search)"

	^ '*', anExpression, '*'. "let's put some wildcards on it"! !

Object subclass: #Person
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

Person subclass: #Author
	instanceVariableNames: 'bio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!Author methodsFor: 'accessing' stamp: 'afv 5/2/2013 20:32'!
bio
	^ bio.! !

!Author methodsFor: 'accessing' stamp: 'afv 5/2/2013 20:32'!
bio: aString
	bio := aString.! !

!Author methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:27'!
initialize
	"This class takes cares of an author entity"
	super initialize.
	
	name := nil.
	bio := nil.! !

!Author methodsFor: 'accessing' stamp: 'afv 5/2/2013 20:32'!
name
	^ name.! !

!Author methodsFor: 'accessing' stamp: 'afv 5/9/2013 22:17'!
name: aString
	name := aString.! !

!Person methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:04'!
initialize
	"Class responsability: Refactored common class from User and Author. Takes care of common fields"

	super initialize.
	name := nil.! !

!Person methodsFor: 'accessing' stamp: 'afv 4/30/2013 22:08'!
name
	^ name.! !

!Person methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:41'!
name: aString 
	name := aString.! !

Person subclass: #User
	instanceVariableNames: 'username password email role status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!User commentStamp: 'afv 5/1/2013 20:44' prior: 0!
User class

Responsability: Represent a single user and its attributes!

!User methodsFor: 'accessing' stamp: 'afv 5/11/2013 22:09'!
doBlock
	status := 'blocked'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 17:43'!
doDisable
	status := 'disabled'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 17:42'!
doEnable
	status := 'enabled'.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 22:00'!
email
	^ email! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:41'!
email: aString 
	email := aString.! !

!User methodsFor: 'testing' stamp: 'afv 5/2/2013 19:06'!
hasEmptyPassword
	"comment stating purpose of message"

	^ password = nil.! !

!User methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:07'!
initialize
	"Class responsability: handles a single user"
	
	super initialize.
	
	"Mandatory user's attributes"
	username := nil.
	password := nil.
	email := nil.
	
	"Automatic attributes (defined by the manager)"
	role := nil.
	status := nil.! !

!User methodsFor: 'testing' stamp: 'afv 5/10/2013 17:38'!
isAdmin
	^ role = 'administrator'.! !

!User methodsFor: 'testing' stamp: 'afv 5/11/2013 22:10'!
isBlocked
	^ status = 'blocked'.! !

!User methodsFor: 'testing' stamp: 'afv 5/11/2013 22:10'!
isDisabled
	^ status = 'disabled'.! !

!User methodsFor: 'testing' stamp: 'afv 5/2/2013 18:31'!
isEnabled
	^ status = 'enabled'.! !

!User methodsFor: 'testing' stamp: 'afv 5/10/2013 17:38'!
isModerator
	^ role = 'moderator'.! !

!User methodsFor: 'testing' stamp: 'afv 5/10/2013 18:28'!
isUser
	^ role = 'user'.! !

!User methodsFor: 'testing' stamp: 'afv 5/2/2013 19:23'!
matchUsername: aUsernameString matchPassword: aPasswordString 
	^ username = aUsernameString and: password = aPasswordString.! !

!User methodsFor: 'testing' stamp: 'afv 5/11/2013 21:17'!
matchesByRole: anExpression
	"Returns true if the didactic material' description matches anExpression"

	^ anExpression match: role.
! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:40'!
password: aString 
	password := aString.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 10:59'!
release
	"Not sure if this is right but we didn't find any way to actualy use the release message in a way that works as we think it should.
	Perhaps this is not a good practice but we intentionally throw the reference to that object and the garbage collector should
	do the rest of the trics (hopefully)"

	self initialize.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:30'!
role
	"comment stating purpose of message"

	^ role.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 22:01'!
status
	^ status.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 15:39'!
username
	^ username.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:39'!
username: aString 
	username := aString.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:16'!
withAdminPrivileges
	role := 'administrator'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:16'!
withModeratorPrivileges
	role := 'moderator'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:17'!
withUserPrivileges
	role := 'user'.! !

Object subclass: #SearchEngine
	instanceVariableNames: 'didacticMaterialsManager resultsByTitle resultsByDescription searchResults usersManager tagsManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!SearchEngine methodsFor: 'accessing' stamp: 'afv 6/8/2013 09:44'!
addFilter: aFilter
	"Applies aFilter to the searchResults and returns the new results OrderedCollection"

	| oldResults results test |
	test := self checkNullResults.
	test ifFalse: [
		oldResults := searchResults veryDeepCopy.
		results := oldResults select: aFilter.
	].
	^ results.
"	searchResults := results"
	! !

!SearchEngine methodsFor: 'testing' stamp: 'it 5/11/2013 13:51'!
checkNullResults
	"comment stating purpose of message"

	| test |
	(test := searchResults = nil) ifTrue: [
		self error: 'You must first search and then add filters to the results.'
	].
	^ test! !

!SearchEngine methodsFor: 'testing' stamp: 'it 5/11/2013 13:57'!
checkNullSearch: input
	"Checks if the filter's input parameter is null/empty and raises an error if so"
	| isInvalid |
	(isInvalid := input isString not) ifFalse: [
		isInvalid := input isEmpty
	].

	isInvalid ifTrue: [self error: 'Invalid search or filter expression'].
	^ isInvalid
! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 5/11/2013 21:20'!
didacticMaterialsManager: aDidacticMaterialsManager
	"Internal API usage only."
	
	didacticMaterialsManager := aDidacticMaterialsManager! !

!SearchEngine methodsFor: 'filtering' stamp: 'afv 6/8/2013 09:59'!
filterbyDidacticMaterialsByStatus: aStatusString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| test searchExpression |
	self checkNullSearch: aStatusString.
	test := self checkNullResults.
	test ifFalse: [
		searchExpression := self wildCardExpression: aStatusString.
		^ [ :didacticMaterial | didacticMaterial matchesByStatus: searchExpression].
	].! !

!SearchEngine methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:05'!
initialize
	"Class responsability: Takes care of searches on the DidacticMaterialsManager, TagsManager and UsersManager singletons instances"
	
	super initialize.
	didacticMaterialsManager := DidacticMaterialsManager singleton.
	usersManager := UsersManager singleton.
	tagsManager := TagsManager singleton.
	
	searchResults := nil.! !

!SearchEngine methodsFor: 'searching' stamp: 'afv 5/11/2013 20:53'!
searchDidacticMaterials: aStringExpression
	"Returns an ordered collection of foun items by aStringExpression by all fields
	Later filters can by applied to narrow search results (as with google for instance)"

	| searchExpression |
	
	searchResults := nil.	
	(self checkNullSearch: aStringExpression) ifFalse: [
		searchExpression := self wildCardExpression: aStringExpression.
		searchResults := didacticMaterialsManager select: [
			:didacticMaterial |
			(searchExpression match: didacticMaterial title) or:
			(searchExpression match: didacticMaterial description)
		].
		
"		resultsByTitle := (didacticMaterialsManager select: [:didacticMaterial | (searchExpression match: didacticMaterial title)]) items.
		resultsByDescription := (didacticMaterialsManager select: [:didacticMaterial | (searchExpression match: didacticMaterial description)]) items."
"		searchResults := (resultsByTitle asOrderedCollection) addAll: (resultsByDescription asOrderedCollection).."
	].
	^ searchResults! !

!SearchEngine methodsFor: 'searching' stamp: 'afv 5/11/2013 20:49'!
searchTags: aStringExpression
	"Searches for Tags that match the following criteria:
	- name matches the informed search string aStringExpression"
		
	| searchExpression |
	
	searchResults := nil.
	
	(self checkNullSearch: aStringExpression) ifFalse: [
		searchExpression := self wildCardExpression: aStringExpression.
		searchResults := tagsManager select: [ :tag | (searchExpression match: tag name) ]
	].
	^ searchResults! !

!SearchEngine methodsFor: 'searching' stamp: 'afv 5/11/2013 20:49'!
searchUsers: aStringExpression
	"Searches for Users that match the following criteria:
	- name matches the informed search string aStringExpression OR
	- username matches aStringExpression OR
	- email matches aStringExpression"
	
	| searchExpression |
	
	searchResults := nil.
	
	(self checkNullSearch: aStringExpression) ifFalse: [
		searchExpression := self wildCardExpression: aStringExpression.
		searchResults := usersManager select: [
			:user |
			(searchExpression match: user name) or:
			(searchExpression match: user username) or:
			(searchExpression match: user email)
		].
	].
	^ searchResults! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 5/11/2013 21:20'!
tagsManager: aTagsManager
	"Internal API usage only."
	
	tagsManager := aTagsManager! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 5/11/2013 21:20'!
usersManager: aUsersManager
	"Internal API usage only."
	
	usersManager := aUsersManager! !

!SearchEngine methodsFor: 'accessing' stamp: 'afv 5/6/2013 20:50'!
wildCardExpression: anExpression
	"Returns an expression surrounded by asterisk (wildcard search)"

	^ '*', anExpression, '*'. "let's put some wildcards on it"! !

Object subclass: #TagObject
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!TagObject methodsFor: 'as yet unclassified' stamp: 'afv 5/11/2013 17:12'!
name
	"comment stating purpose of message"

	^ name.! !

!TagObject methodsFor: 'as yet unclassified' stamp: 'afv 5/11/2013 17:11'!
name: aString
	"comment stating purpose of message"

	name := aString.! !

Object subclass: #TagsManager
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

TagsManager class
	instanceVariableNames: 'uniqueInstance'!

!TagsManager commentStamp: 'afv 5/1/2013 20:44' prior: 0!
TagsManager class

Respnsability: Manage all the tags within the system.

We used a Dictionary as a support structure to store the Tag objects. We choose to use a Dictionary instead of a Set since we pretend to dynamically update the count on DidacticMaterial associated with each tag, so we can have an idea on popular Tags.!

TagsManager class
	instanceVariableNames: 'uniqueInstance'!

!TagsManager class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:04'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of TagsManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/11/2013 00:15'!
add: aTag
	
	self validate: aTag.

	items add: aTag name -> aTag.
	Transcript show: 'Tag doesnt exists on the system, adding it...'.
! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/10/2013 23:40'!
asOrderedCollection
	"Returns an ordered collection containing all tags"

	^ items asOrderedCollection.! !

!TagsManager methodsFor: 'accessing' stamp: 'afv 5/11/2013 15:45'!
at: aTagString.
	"comment stating purpose of message"

	^ items at: aTagString.! !

!TagsManager methodsFor: 'testing' stamp: 'afv 5/11/2013 15:48'!
includesKey: aTagString
	"comment stating purpose of message"

	^ items includesKey: aTagString.! !

!TagsManager methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:06'!
initialize
	"Class responsability: Takes care of tags"
	
	super initialize.
	items := Dictionary new.
	Transcript show: 'TagsManager initialize'.
	! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/11/2013 00:28'!
items
	"Returns the internal structure"

	^ items.! !

!TagsManager methodsFor: 'initialize-release' stamp: 'it 5/11/2013 00:40'!
remove: aTagNameString
	"Removes from the structure aTagNameString. Raises errors if:
	- Tag does not exist"
	
	items removeKey: aTagNameString ifAbsent: [
		self error: 'The tag does not exists on the system, cannot remove it...'.
	].
! !

!TagsManager methodsFor: 'initialize-release' stamp: 'it 5/10/2013 23:46'!
removeAll
	"Internal method only. Not intended to be used as part of the main API"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].
! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/11/2013 18:37'!
select: aBlock
	"Performs a selection according to a code block"

	^ items select: aBlock.! !

!TagsManager methodsFor: 'testing' stamp: 'it 5/11/2013 00:13'!
validate: aTag
	"validates the tag, raising an error if:
	- name is empty
	- name is already being used by another tag"
	
	(aTag name)
		ifNil: [
			self error: 'Tag name cannot be empty!!'.
		] 
		ifNotNil: [
			(items includesKey: (aTag name))
				ifTrue: [
					"There is a Tag with the same name"
					(aTag ~= (items at: (aTag name)))
						ifTrue: [
							self error: 'Another tag with name *', aTag name, '* already exists!!'.
						]
				]
		].
! !

Object subclass: #UsersManager
	instanceVariableNames: 'items emailsMap emailsSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

UsersManager class
	instanceVariableNames: 'uniqueInstance'!

!UsersManager commentStamp: 'afv 5/1/2013 20:47' prior: 0!
UsersManager class

Responsability: Manage all the users on the system.

Restrictions:
	- Username cannot be empty (verified with testEmptyUsername)
	- Password cannot be empty (verified with testEmptyPassword)
	- Email cannot be empty (verified with testEmptyEmail)
	- Role cannot be empty (verified with testEmptyRole)
	- Status cannot be empty (verified with testEmptyStatus)
	- Usernames must be unique (verified via items dictionary [test: testDuplicatedUsername])
	- Emails must be unique (verified via emailsSet [test: testDuplicatedEmails])
	* Test class: UserTests!

UsersManager class
	instanceVariableNames: 'uniqueInstance'!

!UsersManager class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:04'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of UsersManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!UsersManager methodsFor: 'accessing' stamp: 'it 5/10/2013 23:27'!
add: aUser
	"Adds a user to the structure.
	* Test class: UserTests
	"
	self validate: aUser.
	
	items add: (aUser username) -> aUser.
	emailsMap add: (aUser email) -> aUser.
	Transcript show: 'The user doesnt exists on the system, adding it...'; crlf.

! !

!UsersManager methodsFor: 'accessing' stamp: 'afv 5/2/2013 18:01'!
asOrderedCollection
	"comment stating purpose of message"

	^ items asOrderedCollection.! !

!UsersManager methodsFor: 'accessing' stamp: 'it 5/11/2013 19:44'!
doLogin: aUsernameString password: aPasswordString
	"Authenticates the user into the system.
	Returns true if username/password matches the ones provided, false if not.
	Possible errors:
	- Username missmatch. Test: testDoLoginWithNonExistingUsername
	- Password missmatch. Test: testDoLoginWithWrongPassword
	- User is disabled. Test:
	"
	| user test acceptedAuth msg error |
	user := nil.
	test := items includesKey: aUsernameString.
	(test) ifFalse: [
		msg := 'User doesnt exist'.
		error := true.
	].
	(test) ifTrue: [
		user := items at: aUsernameString.
		error := false.
		acceptedAuth := user matchUsername: aUsernameString matchPassword: aPasswordString.
		acceptedAuth ifFalse: [
			msg := 'Wrong password. No cookies for you!!'.
			error := true.
		].
		acceptedAuth ifTrue: [
			acceptedAuth := acceptedAuth and: user isEnabled.
			acceptedAuth ifFalse: [
				msg := 'The user is disabled, you cannot login.'.
				error := true.
			].
			acceptedAuth ifTrue: [
				msg := 'User loged in'.
			].
		].
	].
	Transcript show: msg; cr.
	(error) ifTrue: [
		PermissionError signal: msg.
	].
	^ user.! !

!UsersManager methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:08'!
initialize
	"Class responsability: handles several users and encapsulates authentication methods"
	
	super initialize.
	items := Dictionary new.
	emailsMap := Dictionary new.
	Transcript show: 'UserManager initialize'; crlf.! !

!UsersManager methodsFor: 'accessing' stamp: 'afv 5/10/2013 10:47'!
items
	"Returns the internal structure"

	^ items.! !

!UsersManager methodsFor: 'initialize-release' stamp: 'it 5/11/2013 14:36'!
remove: aUsername
	"Removes from the structure aUsername. Raises errors on:
	- Non existing aUsername (verified with testEmptyUsername)"
	
	| user |
	Transcript show: 'Removing *', aUsername, '* from the database'; crlf.
	
	user := items at: aUsername ifAbsent: [
		self error: 'The user does not exists on the system, cannot remove it...'.
		^ self
	].

	items removeKey: aUsername.
	emailsMap removeKey: (user email) ifAbsent: [].
! !

!UsersManager methodsFor: 'accessing' stamp: 'afv 5/11/2013 12:34'!
removeAll
	"Internal method only. Not intended to be used as part of the main API"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].

	emailsMap keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		emailsMap removeKey: key.
	].! !

!UsersManager methodsFor: 'accessing' stamp: 'it 5/11/2013 14:02'!
select: aFilterBlock
	"Performs a Collection select using the passed in argument. Returns the resulting collection."

	^ items select: aFilterBlock.
	! !

!UsersManager methodsFor: 'testing' stamp: 'it 5/11/2013 19:34'!
validate: aUser
	"Validates the passed-in User and raises errors on:
	- empty username (verified with testEmptyUsername)
	- empty password (verified with testEmptyPassword)
	- empty email (verified with testEmptyEmail)
	- empty role (verified with testEmptyRole)
	- empty status (verified with testEmptyStatus)
	- repeated usernames (verified via items dictionary [test: testDuplicatedUsername])
	- repeated emails (verified via emailsMap [test: testDuplicatedEmails])
	"
	
	(aUser username) ifNil: [
		ValidationError signal: 'Username cannot be empty!!'.
	].

	(aUser hasEmptyPassword) ifTrue: [
		ValidationError signal: 'Password cannot be empty Mr. genious!!'.
	].

	(aUser name) ifNil: [
		ValidationError signal: 'Name cannot be empty!!'.
	].

	(aUser email) ifNil: [
		ValidationError signal: 'Email cannot be empty!!'.
	].

	(aUser role) ifNil: [
		ValidationError signal: 'Role cannot be empty!!'.
	].

	(aUser status) ifNil: [
		ValidationError signal: 'Status cannot be empty oh oh!!'.
	].

	(emailsMap includesKey: (aUser email)) ifTrue: [
		"There is a User with the same email!!
		Checks if its not the same instance. Raise an error only if it's a different user"
		(aUser ~= (emailsMap at: (aUser email))) ifTrue: [
			ValidationError signal: 'A user with that email address already exists in the system!!'.
		]
	].

	(items includesKey: (aUser username)) ifTrue: [
		"There is a User with the same username!!
		Checks if its not the same instance. Raise an error only if it's a different user"
		(aUser ~= (items at: (aUser username))) ifTrue: [
			ValidationError signal: 'Cannot add that username, it already exists on the system. Did you forget the password? No cookies for you!!'.
		]
	].
! !

Error subclass: #PermissionError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

Error subclass: #ValidationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!
